levelbar .strength-weak {
  background-color: #cc0000;
  border-color: #cc0000;
}

levelbar .strength-low {
  background-color: #f5ce00;
  border-color: #f5ce00;
}

levelbar .strength-medium,
levelbar .strength-good,
levelbar .strength-high {
  background-color: #73d216;
  border-color: #73d216;
}
