/* Styles borrowed from GTK 4
 * https://gitlab.gnome.org/GNOME/gtk/blob/672d7f679adf543785042ab45d7e59688103464c/gtk/theme/Adwaita/_common.scss#L287-327
 */
.large-title {
   font-weight: 300;
   font-size: 24pt;
   letter-spacing: 0.2rem;
}

.title-1 {
   font-weight: 800;
   font-size: 20pt;
}
